use axum::extract::FromRequestParts;
use axum::http::{header, request::Parts, HeaderMap, HeaderValue, StatusCode};
use tower_sessions::Session;

#[allow(dead_code)] // TODO: WIP
pub struct AuthenticatedUser {
    session: Session,
    api_token: String,
}

pub const TOKEN_KEY: &str = "API_TOKEN";

impl<S> FromRequestParts<S> for AuthenticatedUser
where
    S: Send + Sync,
{
    type Rejection = (StatusCode, HeaderMap, String);

    async fn from_request_parts(req: &mut Parts, state: &S) -> Result<Self, Self::Rejection> {
        let session = match Session::from_request_parts(req, state).await {
            Ok(session) => session,
            Err((status, body)) => return Err((status, HeaderMap::new(), body.to_string())),
        };

        let api_token: Option<String> = session.get(TOKEN_KEY).await.unwrap(); // TODO: is it safe to [unwrap] here?

        match api_token {
            Some(api_token) => Ok(Self { session, api_token }),
            None => {
                // Unclear that this case can actually happen, but in doubt,
                // clear the session and force to re-auth.
                // It is safe to [unwrap] the result here because we use
                // [MemoryStore] as the underlying store, on which [delete]
                // never fails.
                session.flush().await.unwrap();
                Err((
                    StatusCode::SEE_OTHER,
                    HeaderMap::from_iter(
                        [(header::LOCATION, HeaderValue::from_static("/auth"))].into_iter(),
                    ),
                    "client is not authenticated, redirecting to /auth".to_string(),
                ))
            }
        }
    }
}
